@@ -220,15 +220,28 @@ load_status_changed_cb (WebKitWebView *view,
                            FALSE,
                            FALSE);
 
-    /*
-     * FIXME: as a temporary workaround while soup lacks the needed
-     * security API, determine security level based on the existence of
-     * a 'https' prefix for the URI
-     */
-    if (uri && g_str_has_prefix (uri, "https"))
-      security_level = EPHY_WEB_VIEW_STATE_IS_SECURE_HIGH;
-    else
+#ifdef GTLS_SYSTEM_CA_FILE
+    if (uri && g_str_has_prefix (uri, "https")) {
+      WebKitWebFrame *frame;
+      WebKitWebDataSource *source;
+      WebKitNetworkRequest *request;
+      SoupMessage *message;
+
+      frame = webkit_web_view_get_main_frame (view);
+      source = webkit_web_frame_get_data_source (frame);
+      request = webkit_web_data_source_get_request (source);
+      message = webkit_network_request_get_message (request);
+
+      if (message &&
+          (soup_message_get_flags (message) & SOUP_MESSAGE_CERTIFICATE_TRUSTED))
+        security_level = EPHY_WEB_VIEW_STATE_IS_SECURE_HIGH;
+      else
+        security_level = EPHY_WEB_VIEW_STATE_IS_BROKEN;
+    } else
       security_level = EPHY_WEB_VIEW_STATE_IS_UNKNOWN;
+#else
+    security_level = EPHY_WEB_VIEW_STATE_IS_UNKNOWN;
+#endif
 
     ephy_web_view_set_security_level (EPHY_WEB_VIEW (view), security_level);
   } else if (status == WEBKIT_LOAD_PROVISIONAL || status == WEBKIT_LOAD_FINISHED) {
@@ -561,6 +574,8 @@ define_destination_uri (WebKitDownload *download,
 
   destination_uri = g_strconcat ("file://", destination_filename, NULL);
 
+  LOG ("define_destination_uri: Downloading to %s", destination_filename);
+
   webkit_download_set_destination_uri (download, destination_uri);
 
   g_free (tmp_dir);
@@ -718,6 +733,7 @@ confirm_action_from_mime (WebKitWebView *web_view,
                                                 "it could potentially damage your documents or "
                                                 "invade your privacy. You can download it instead."),
                                               mime_description, suggested_filename);
+    action_label = STOCK_DOWNLOAD;
   } else if (action == DOWNLOAD_ACTION_OPEN && helper_app) {
     dialog = gtk_message_dialog_new (GTK_WINDOW (parent_window),
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -779,7 +795,7 @@ confirm_action_from_mime (WebKitWebView *web_view,
 static void
 download_status_changed_cb (GObject *object,
                             GParamSpec *pspec,
-                            WebKitWebView *web_view)
+                            EphyEmbed *embed)
 {
   WebKitDownload *download = WEBKIT_DOWNLOAD (object);
 
@@ -794,9 +810,11 @@ download_status_changed_cb (GObject *object,
     destination_uri = g_object_get_data (G_OBJECT (download),
                                          "user-destination-uri");
 
+    LOG ("download_status_changed_cb: finished, moving temp file %s to %s",
+         temp_uri, destination_uri);
+
     /* No user-destination-uri is set, hence this is an auto download and we
-     * have nothing else to do.
-     */
+     * have nothing else to do. */
     if (destination_uri == NULL) return;
 
     temp = g_file_new_for_uri (temp_uri);
@@ -826,8 +844,11 @@ download_status_changed_cb (GObject *object,
       return;
     }
 
-    g_object_ref (download); /* balanced in confirm_action_response_cb */
-    confirm_action_from_mime (web_view, download, DOWNLOAD_ACTION_DOWNLOAD);
+    /* Balanced in confirm_action_response_cb. */
+    g_object_ref (download);
+
+    confirm_action_from_mime (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed),
+                              download, DOWNLOAD_ACTION_DOWNLOAD);
   }
 }
 
@@ -836,15 +857,18 @@ download_error_cb (WebKitDownload *download,
                    gint error_code,
                    gint error_detail,
                    const gchar *reason,
-                   WebKitWebView *view)
+                   EphyEmbed *embed)
 {
-  /* FIXME: handle download errors and notify the user */
+  /* FIXME: handle download errors and notify the user. */
+  LOG ("download_error_cb: Error (%d:%d): %s", error_code, error_detail, reason);
+
   return FALSE;
 }
 
 static gboolean
 download_requested_cb (WebKitWebView *web_view,
-                       WebKitDownload *download)
+                       WebKitDownload *download,
+                       EphyEmbed *embed)
 {
   /* Is download locked down? */
   if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_SAVE_TO_DISK))
@@ -852,12 +876,12 @@ download_requested_cb (WebKitWebView *web_view,
 
   /* Wait for the request to be sent in all cases, so that we have a
    * response which may contain a suggested filename */
-  g_signal_connect_object (download, "notify::status",
-                           G_CALLBACK (download_status_changed_cb),
-                           web_view, 0);
-  g_signal_connect_object (download, "error",
-                           G_CALLBACK (download_error_cb),
-                           web_view, 0);
+  g_signal_connect (download, "notify::status",
+                    G_CALLBACK (download_status_changed_cb),
+                    embed);
+  g_signal_connect (download, "error",
+                    G_CALLBACK (download_error_cb),
+                    embed);
 
   /* If we are not performing an auto-download, we will ask the user
    * where they want the file to go to; we will start downloading to a
