@@ -14,17 +14,20 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-dialog.h 6589 2006-09-13 11:49:01Z chpe $
  */
 
+#if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
+#error "Only <epiphany/epiphany.h> can be included directly."
+#endif
+
 #ifndef EPHY_DIALOG_H
 #define EPHY_DIALOG_H
 
 #include <glib-object.h>
 #include <glib.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
@@ -38,6 +41,7 @@ G_BEGIN_DECLS
 typedef struct _EphyDialogClass		EphyDialogClass;
 typedef struct _EphyDialog		EphyDialog;
 typedef struct _EphyDialogPrivate	EphyDialogPrivate;
+typedef struct _EphyDialogProperty	EphyDialogProperty;
 
 typedef enum
 {
@@ -46,13 +50,13 @@ typedef enum
 	PT_INVERTED	= 1 << 1
 } EphyDialogApplyType;
 
-typedef struct
+struct _EphyDialogProperty
 {
 	const char *id;
 	const char *pref;
 	EphyDialogApplyType apply_type;
 	GType data_type;
-} EphyDialogProperty;
+};
 
 struct _EphyDialogClass
 {
@@ -93,13 +97,13 @@ void		ephy_dialog_construct		(EphyDialog *dialog,
 						 const char *domain);
 
 void		ephy_dialog_add_enum		(EphyDialog *dialog,
-						 const char *id,
+						 const char *property_id,
 						 guint n_items,
 						 const char * const *items);
 
 void		ephy_dialog_set_data_column	(EphyDialog *dialog,
-						 const char *id,
-						 int col);
+						 const char *property_id,
+						 int column);
 
 void		ephy_dialog_set_size_group	(EphyDialog *dialog,
 						 const char *first_id,
@@ -123,7 +127,7 @@ GtkWidget      *ephy_dialog_get_control		(EphyDialog *dialog,
 						 const char *property_id);
 
 void		ephy_dialog_get_controls	(EphyDialog *dialog,
-						 const char *first_property_id,
+						 const char *first_id,
 						 ...);
 
 gboolean	ephy_dialog_get_value		(EphyDialog *dialog,
