@@ -34,14 +34,22 @@
 #include <gtk/gtk.h>
 
 #include <string.h>
+
+/**
+ * SECTION:ephy-location-action
+ * @short_description: An #EphyLinkAction implementation
+ *
+ * #EphyLocationAction handles navigation together with #EphyLocationEntry
+ */
+
 #define EPHY_LOCATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_LOCATION_ACTION, EphyLocationActionPrivate))
 
 struct _EphyLocationActionPrivate
 {
 	EphyWindow *window;
+	GtkWidget *proxy;
 	GList *actions;
 	char *address;
-	char *typed_address;
 	EphyNode *smart_bmks;
 	EphyBookmarks *bookmarks;
 	GdkPixbuf *icon;
@@ -166,7 +174,7 @@ action_activated_cb (GtkEntryCompletion *completion,
 		if (url == NULL) return;
 
 		ephy_link_open (EPHY_LINK (action), url, NULL,
-				ephy_link_flags_from_current_event () | EPHY_LINK_ALLOW_FIXUP);
+				ephy_link_flags_from_current_event ());
 
 		g_free (url);
 	}
@@ -180,12 +188,14 @@ entry_activate_cb (GtkEntry *entry,
 	const char *content;
 	char *address;
 	GtkAction *action;
+	EphyLocationActionPrivate *priv;
 	
 	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+	priv = EPHY_LOCATION_ACTION (action)->priv;
 
-	if (EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked)
+	if (priv->sync_address_is_blocked)
 	{
-		EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked = FALSE;
+		priv->sync_address_is_blocked = FALSE;
 		g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
 	}	
 
@@ -198,7 +208,7 @@ entry_activate_cb (GtkEntry *entry,
 	g_return_if_fail (address != NULL);
 
 	ephy_link_open (EPHY_LINK (action), g_strstrip (address), NULL, 
-			ephy_link_flags_from_current_event () | EPHY_LINK_ALLOW_FIXUP);
+			ephy_link_flags_from_current_event ());
 
 	g_free (address);
 }
@@ -213,7 +223,7 @@ user_changed_cb (GtkWidget *proxy, EphyLocationAction *action)
 	LOG ("user_changed_cb, new address %s", address);
 
 	g_signal_handlers_block_by_func (action, G_CALLBACK (sync_address), proxy);
-	ephy_location_action_set_address (action, address, NULL);
+	ephy_location_action_set_address (action, address);
 	g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
 }
 
@@ -236,8 +246,7 @@ sync_address (GtkAction *gaction,
 	LOG ("sync_address %s", action->priv->address);
 
 	g_signal_handlers_block_by_func (proxy, G_CALLBACK (user_changed_cb), action);
-	ephy_location_entry_set_location (lentry, priv->address,
-					  priv->typed_address);
+	ephy_location_entry_set_location (lentry, priv->address);
 	g_signal_handlers_unblock_by_func (proxy, G_CALLBACK (user_changed_cb), action);
 }
 
@@ -324,7 +333,7 @@ get_location_cb (EphyLocationEntry *entry,
 	embed = ephy_embed_container_get_active_child 
 	  (EPHY_EMBED_CONTAINER (priv->window));
 
-	return g_strdup (ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)));
+	return g_strdup (ephy_web_view_get_address (ephy_embed_get_web_view (embed)));
 }
 
 static char *
@@ -336,7 +345,7 @@ get_title_cb (EphyLocationEntry *entry,
 	embed = ephy_embed_container_get_active_child 
 	  (EPHY_EMBED_CONTAINER (action->priv->window));
 
-	return g_strdup (ephy_web_view_get_title (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)));
+	return g_strdup (ephy_web_view_get_title (ephy_embed_get_web_view (embed)));
 }
 
 static void
@@ -396,11 +405,14 @@ focus_in_event_cb (GtkWidget *entry,
 		   GtkWidget *proxy)
 {
 	GtkAction *action;
+	EphyLocationActionPrivate *priv;
+
 	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
 
-	if (!EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked)
+	priv = EPHY_LOCATION_ACTION (action)->priv;
+	if (!priv->sync_address_is_blocked)
 	{		
-		EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked = TRUE;
+		priv->sync_address_is_blocked = TRUE;
 		g_signal_handlers_block_by_func (action, G_CALLBACK (sync_address), proxy);
 	}
 	
@@ -413,17 +425,37 @@ focus_out_event_cb (GtkWidget *entry,
 		    GtkWidget *proxy)
 {
 	GtkAction *action;
-	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+	EphyLocationActionPrivate *priv;
 
-	if (EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked)
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+	priv = EPHY_LOCATION_ACTION (action)->priv;
+
+	if (priv->sync_address_is_blocked)
 	{
-		EPHY_LOCATION_ACTION (action)->priv->sync_address_is_blocked = FALSE;
+		priv->sync_address_is_blocked = FALSE;
 		g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
 	}
 	
 	return FALSE;
 }
 
+static void
+switch_page_cb (GtkNotebook *notebook,
+		GtkNotebookPage *page,
+		guint page_num,
+		GtkAction *action)
+{
+	EphyLocationActionPrivate *priv;
+
+	priv = EPHY_LOCATION_ACTION (action)->priv;
+
+	if (priv->sync_address_is_blocked == TRUE)
+	{
+		priv->sync_address_is_blocked = FALSE;
+		g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), priv->proxy);
+	}
+}
+
 static void
 connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
@@ -432,6 +464,16 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 		EphyCompletionModel *model;
 		GtkWidget *entry;
+		GtkWidget *notebook;
+		EphyLocationActionPrivate *priv;
+
+		priv = EPHY_LOCATION_ACTION (action)->priv;
+		priv->proxy = proxy;
+
+		notebook = ephy_window_get_notebook (priv->window);
+
+		g_signal_connect (notebook, "switch-page",
+				  G_CALLBACK (switch_page_cb), action);
 
 		model = ephy_completion_model_new ();
 		ephy_location_entry_set_completion (EPHY_LOCATION_ENTRY (proxy),
@@ -503,9 +545,14 @@ disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 	if (EPHY_IS_LOCATION_ENTRY (proxy))
 	{
 		EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
-		GtkWidget *entry;
+		GtkWidget *entry, *notebook;
+		EphyLocationActionPrivate *priv;
+
+		priv = EPHY_LOCATION_ACTION (action)->priv;
+		priv->proxy = NULL;
 
 		entry = ephy_location_entry_get_entry (lentry);
+		notebook = ephy_window_get_notebook (priv->window);
 
 		g_signal_handlers_disconnect_matched (action, G_SIGNAL_MATCH_DATA,
 						      0, 0, NULL, NULL, proxy);
@@ -513,6 +560,8 @@ disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 						      0, 0, NULL, NULL, action);
 		g_signal_handlers_disconnect_matched (entry, G_SIGNAL_MATCH_DATA,
 						      0, 0, NULL, NULL, action);
+		g_signal_handlers_disconnect_matched (notebook, G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL, NULL, action);
 	}
 }
 
@@ -528,7 +577,7 @@ ephy_location_action_set_property (GObject *object,
 	switch (prop_id)
 	{
 		case PROP_ADDRESS:
-			ephy_location_action_set_address (action, g_value_get_string (value), NULL);
+			ephy_location_action_set_address (action, g_value_get_string (value));
 			break;
 		case PROP_EDITABLE:
 			priv->editable = g_value_get_boolean (value);
@@ -612,6 +661,13 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 	action_class->connect_proxy = connect_proxy;
 	action_class->disconnect_proxy = disconnect_proxy;
 
+	/**
+	* EphyLocationAction::lock-clicked:
+	* @action: the object which received the signal.
+	*
+	* Emitted when the user clicks on the security icon of the internal
+	* #EphyLocationEntry.
+	*/
 	signals[LOCK_CLICKED] = g_signal_new (
 		"lock-clicked",
 		EPHY_TYPE_LOCATION_ACTION,
@@ -622,66 +678,107 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 		G_TYPE_NONE,
 		0);
 
+	/**
+	* EphyLocationAction:address:
+	*
+	* The address of the current location.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_ADDRESS,
 					 g_param_spec_string ("address",
 							      "Address",
-							      "The address",
+							      "The address of the current location",
 							      "",
 							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	/**
+	* EphyLocationAction:editable:
+	*
+	* Whether the location bar entry can be edited.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_EDITABLE,
 					 g_param_spec_boolean ("editable",
 							       "Editable",
-							       "Editable",
+							       "Whether the location bar entry can be edited",
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+	/**
+	* EphyLocationAction:icon:
+	*
+	* The icon corresponding to the current location.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
 					 g_param_spec_object ("icon",
 							      "Icon",
-							      "The icon",
+							      "The icon corresponding to the current location",
 							      GDK_TYPE_PIXBUF,
 							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+	/**
+	* EphyLocationAction:lock-stock-id:
+	*
+	* Stock id of the security icon.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_LOCK_STOCK,
 					 g_param_spec_string  ("lock-stock-id",
 							       "Lock Stock ID",
-							       "Lock Stock ID",
+							       "Stock id of the security icon",
 							       NULL,
 							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+	/**
+	* EphyLocationAction:lock-tooltip:
+	*
+	* Tooltip for the security icon.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_LOCK_TOOLTIP,
 					 g_param_spec_string  ("lock-tooltip",
 							       "Lock Tooltip",
-							       "The icon",
+							       "Tooltip for the security icon",
 							       NULL,
 							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+	/**
+	* EphyLocationAction:secure:
+	*
+	* Whether the current page is loaded over a secure connection.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_SECURE,
 					 g_param_spec_boolean ("secure",
 							       "Secure",
-							       "Secure",
+							       "Whether the current page is loaded over a secure connection",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+	/**
+	* EphyLocationAction:show-lock:
+	*
+	* If we should show the security icon.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_LOCK,
 					 g_param_spec_boolean ("show-lock",
 							       "Show Lock",
-							       "Show Lock",
+							       "If we should show the security icon",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+	/**
+	* EphyLocationAction:window:
+	*
+	* The parent window.
+	*/
 	g_object_class_install_property (object_class,
 					 PROP_WINDOW,
 					 g_param_spec_object ("window",
 							      "Window",
-							      "The navigation window",
+							      "The parent window",
 							      G_TYPE_OBJECT,
 							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
 							      G_PARAM_CONSTRUCT_ONLY));
@@ -836,13 +933,20 @@ ephy_location_action_finalize (GObject *object)
 
 	g_list_free (priv->actions);
 	g_free (priv->address);
-	g_free (priv->typed_address);
 	g_free (priv->lock_stock_id);
 	g_free (priv->lock_tooltip);
 
 	G_OBJECT_CLASS (ephy_location_action_parent_class)->finalize (object);
 }
 
+/**
+ * ephy_location_action_get_address:
+ * @action: an #EphyLocationAction
+ *
+ * Retrieves the currently loaded address.
+ *
+ * Returns: the current address
+ **/
 const char *
 ephy_location_action_get_address (EphyLocationAction *action)
 {
@@ -851,10 +955,16 @@ ephy_location_action_get_address (EphyLocationAction *action)
 	return action->priv->address;
 }
 
+/**
+ * ephy_location_action_set_address:
+ * @action: an #EphyLocationAction
+ * @address: new address
+ *
+ * Sets @address as the address of @action.
+ **/
 void
 ephy_location_action_set_address (EphyLocationAction *action,
-				  const char *address,
-				  const char *typed_address)
+				  const char *address)
 {
 	EphyLocationActionPrivate *priv;
 
@@ -867,8 +977,5 @@ ephy_location_action_set_address (EphyLocationAction *action,
 	g_free (priv->address);
 	priv->address = g_strdup (address);
 
-	g_free (priv->typed_address);
-	priv->typed_address = typed_address ? g_strdup (typed_address) : NULL;
-
 	g_object_notify (G_OBJECT (action), "address");
 }
