@@ -104,10 +104,9 @@ update_location_editable (EphyWindow *window,
 		/* embed is NULL on startup */
 		if (embed != NULL)
 		{
-			address = ephy_web_view_get_location (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), TRUE);
-			ephy_toolbar_set_location (EPHY_TOOLBAR (toolbar), address, NULL);
-			ephy_web_view_set_typed_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), NULL,
-							 EPHY_WEB_VIEW_ADDRESS_EXPIRE_CURRENT);
+			address = ephy_web_view_get_location (ephy_embed_get_web_view (embed), TRUE);
+			ephy_toolbar_set_location (EPHY_TOOLBAR (toolbar), address);
+			ephy_web_view_set_typed_address (ephy_embed_get_web_view (embed), NULL);
 			g_free (address);
 		}
 	}
